// Get all the questions
const quiz_questions = Array.from(document.querySelectorAll('.question')) || [];
quiz_questions.forEach((question) => {
    // Create a button and append it to the header of the question
    const wiz_show_answers_button_html = `
        <button 
        class="wiz-style wiz-ui"
        style="
            margin-left: 8px !important;
            border-radius: 99px !important;
            background-color: #1A2426 !important;
            border: 2px solid #00E6D2 !important;
            color: white !important;
            padding: 6px 16px !important;
            font-size: 14px !important;
            font-weight: 600 !important;
            margin: 0px 12px !important;
            transition-duration: 200ms !important;
            display: inline-flex !important;
            align-items: center; !important"
            type="button"
            >
            <img src="${chrome.runtime.getURL('img/logo.webp')}" alt="Show Answers CanvasWizard" style="height: 17px; width: auto; margin-right: 12px;">
            Show Answer
        </button>
    `;
    const temp_div = document.createElement('div');
    temp_div.innerHTML = wiz_show_answers_button_html;
    const wiz_show_answers_button = temp_div.firstElementChild;
    question.querySelector('.header').appendChild(wiz_show_answers_button); 
    
    // Add event listener to the button within the same scope
    wiz_show_answers_button.addEventListener('click', (event) => {
        event.preventDefault();
        event.stopPropagation();

        // Remove the .wiz-answer-dialogue elements
        question.querySelector('.wiz-answer-dialogue')?.remove();

        // If the question contains an image, show the screenshot modal instead.
        if(question.querySelector('.text').outerHTML.includes('<img')){
            chrome.storage.local.get(['wiz_settings']).then(result => {
                chrome.storage.local.set({ 
                    wiz_settings: {
                        ...result.wiz_settings,
                        screenshot_visible: true,
                        chat_visible: false,
                        history_visible: false,
                    }
                });
            });
            return;
        }

        const loading_messages = ["Let me think...", "I'm on it...", "That's a good one...", "I gotchu...", "Just a sec..."];
        const random_messages = loading_messages[Math.floor(Math.random() * loading_messages.length)];
        
        // Remove the image and update button content with spinner
        wiz_show_answers_button.innerHTML = `
            <svg style="margin-right: 8px;  width: 16px;  height: 16px;" class="animate-spin  text-wiz-mint" xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 24">
                <circle style="opacity: 0.25;" cx="12" cy="12" r="10" stroke="currentColor" stroke-width="4"></circle>
                <path style="opacity: 0.75;" fill="currentColor" d="M4 12a8 8 0 018-8V0C5.373 0 0 5.373 0 12h4zm2 5.291A7.962 7.962 0 014 12H0c0 3.042 1.135 5.824 3 7.938l3-2.647z"></path>
            </svg>
            ${random_messages}
        `;

        const wiz_answer_dialogue = document.createElement('div');

        chrome.storage.local.get('wiz_settings', function(result) {
            extension_uuid = result.wiz_settings?.extension_uuid || '';
        
            fetch(`${API_ENDPOINT}/answer`, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                    'Extension-UUID': extension_uuid,
                },
                body: JSON.stringify({
                    question_html: question.outerHTML,
                })
            })
            .then(response => {
                if (!response.ok) {throw new Error(response.status);}
                return response.json();
            })
            .then(data => {
                // Get existing history and append new entry
                chrome.storage.local.get(['wiz_history'], (result) => {
                    const history = result.wiz_history || [];
                    history.unshift({
                        uuid: data.data.uuid,
                        question: data.data.question,
                        answer: data.data.answer,
                        explanation: data.data.explanation,
                    });
                    // Store updated history
                    chrome.storage.local.set({ wiz_history: history });
                });

                wiz_answer_dialogue.style.cssText = `padding: 12px;`;
                wiz_answer_dialogue.className = 'wiz-answer-dialogue wiz-ui';
                wiz_answer_dialogue.innerHTML = `
                    <div class="wiz-style" style="background-color: #1E1E1E !important; border: 2px solid #00E6D2; border-radius: 4px; width: 100%; height: auto; display: flex; flex-direction: column;">
                        <div style="pointer-events: none; color: #2c3b44; background-color: #f5f5f5; font-weight: normal; padding: 8px; font-size: 14px;">
                            <p class="wiz-question-answer" style="line-height: 1; margin: 0; padding: 0;"><strong style="font-weight: 600; margin-right: 4px;">Correct Answer(s):</strong> ${data.data.answer}</p>
                            <p class="explanation-header" style="font-weight: 600; margin-bottom: 4px;">Explanation:</p>
                            <div class="wiz-question-explanation" style="font-size: 13px; line-height: 1.4; margin-bottom: 8px;">${data.data.explanation}</div>
                            <p class="disclaimer" style="font-size: 12px; color: #666; font-style: italic; margin-bottom: 12px; line-height: 1.1;">The explanation provided above is AI-generated and may not be entirely accurate. Please verify with trusted sources.</p>
                            <div class="watermark-container" style="display: flex; justify-content: flex-end;">
                                <img src=${chrome.runtime.getURL('img/watermark.png')} alt="Watermark by CanvasWizard.io" style="height: 24px; width: auto;">
                            </div>
                        </div>
                    </div>
                `;

                const header = question.querySelector('.header');
                header.insertAdjacentElement('afterend', wiz_answer_dialogue);

                // Set a timer to remove the answer and the show answer button after 10 seconds
                setTimeout(() => {
                    wiz_answer_dialogue.remove();
                    wiz_show_answers_button.style.display = 'none'; // Hide the "Show Answer" button
                    // Optionally, show the icon next to the button for 10 seconds
                    wiz_show_answers_button.innerHTML = `
                        <svg style="width: 20px; height: 20px; margin-right: 10px" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24">
                            <path fill="currentColor" d="M12 13.825L8.1 17.7q-.275.275-.687.288T6.7 17.7q-.275-.275-.275-.7t.275-.7l4.6-4.6q.15-.15.325-.213t.375-.062t.375.062t.325.213l4.6 4.6q.275.275.288.688t-.288.712q-.275.275-.7.275t-.7-.275zm0-6L8.1 11.7q-.275.275-.687.288T6.7 11.7q-.275-.275-.275-.7t.275-.7l4.6-4.6q.15-.15.325-.212T12 5.425t.375.063t.325.212l4.6 4.6q.275.275.288.688t-.288.712q-.275.275-.7.275t-.7-.275z"/>
                        </svg>
                        Answer Hidden
                    `;
                }, 10000); // 10 seconds delay

            })  
            .catch(error => {
                if (error instanceof Error && error.message) {
                    const matches = error.message.match(/\d+/);
                    if (matches && matches[0]) {
                        const statusCode = parseInt(matches[0]);
                        if (statusCode === 401) {
                            error_message = "Your account is not activated. Please activate it by visiting the setup page or clicking the extension icon.";
                        } else {
                            error_message = "An error occurred while processing this question. Please try again.";
                        }
                    } else {
                        error_message = "An error occurred while processing this question. Please try again.";
                    }
                } else {
                    error_message = "An unexpected error occurred. Please try again.";
                }
            })
            .finally(() => {
                // Restore button content after the process
                wiz_show_answers_button.innerHTML = `
                    <img src="${chrome.runtime.getURL('img/logo.webp')}" alt="Show Answers CanvasWizard" style="height: 17px; width: auto; margin-right: 12px;">
                    Show Answer
                `;
                
                // Update styles instead of recreating the button
                Object.assign(wiz_show_answers_button.style, {
                    marginLeft: '8px',
                    borderRadius: '99px',
                    backgroundColor: '#1A2426',
                    border: '2px solid #00E6D2',
                    color: 'white',
                    padding: '6px 16px',
                    fontSize: '14px',
                    fontWeight: '600',
                    margin: '0px 12px',
                    transitionDuration: '200ms',
                    display: 'inline-flex',
                    alignItems: 'center'
                });
            });
        });
    });
});
