// Init the base settings for the extension
chrome.runtime.onInstalled.addListener(() => {
    chrome.storage.local.get(['wiz_settings']).then(result => {
        if (!result.wiz_settings) {
            // Get or generate extension UUID and initialize settings in one chain
            return chrome.storage.local.get('extensionUUID') // Legacy Extension Version support while migrating.
                .then(result => {
                    const extension_uuid = result.extensionUUID || crypto.randomUUID();
                    return chrome.storage.local.clear()
                        .then(() => chrome.storage.local.set({
                            wiz_settings: {
                                incognito_mode: false,
                                sidebar_visible: true,
                                screenshot_visible: false,
                                chat_visible: false,
                                history_visible: false,
                                extension_uuid: extension_uuid
                            }
                        }))
                        .catch(err => console.error('Error clearing storage:', err));
                });
        }
    });

    // Delete all wiz_notes from storage
    chrome.storage.local.remove('wiz_notes').catch(err => console.error('Error removing Wiz Notes.', err));
});

// Capture Screenshot Element
chrome.runtime.onMessage.addListener(
    function(request, sender, sendResponse) {
        if (request.msg === 'capture') {
            chrome.tabs.captureVisibleTab(
                null,
                {},
                function(dataUrl) {
                    sendResponse({screenshot:dataUrl});
                }
            );
        }
        return true;
    }
);

chrome.runtime.setUninstallURL('https://www.canvaswizard.co/uninstall');
chrome.runtime.onInstalled.addListener(function (details) {
    if (details.reason == 'install') {chrome.tabs.create({url: 'https://www.canvaswizard.co/onboarding'});}
});


  