function loadHistory() {
    // Get access to the chrome localstorage and get the chat history
    chrome.storage.local.get('wiz_history', function (result) {
        // Get the chat history from the local storage
        const previous_answers_history = result.wiz_history || [];
        // Get the chat history container
        const wiz_history_container = document.getElementById('wiz-history-container');
        const wiz_history_scroll_container = document.getElementById('wiz-history-scroll-container');
        // Scroll to the top of the chat history scroll container
        wiz_history_scroll_container.scrollTop = 0;
        // Clear the chat history container
        wiz_history_container.innerHTML = '';

        // Iterate through the chat history and add each message to the chat history container
        previous_answers_history.forEach(function (element) {
            wiz_history_container.innerHTML += `
            <div style="
                padding: 10px;
                border-radius: 4px;
                line-height: 1.25;
                max-width: 100%;
                list-style-type: disc;
                font-size: 14px;
                background-color: #ffffff;
                border: 1px solid rgb(0, 230, 210);
                color: white;
                align-self: flex-start;
                margin-bottom: 13px !important;">
                <div style="pointer-events: none; color: #2c3b44; background-color: #ffffff; font-weight: normal; padding: 2px; font-size: 14px;">
                    <p class="wiz-question" style="line-height: 1; margin: 0; padding: 0;">
                        <strong style="font-weight: 600; margin-right: 4px;">Question:</strong> 
                    </p>

                    ${element.question.startsWith('data:image/png;base64') 
                        ? `<img style="border-radius: 2px;" src="${element.question}">`
                        : `<p>${element.question}</p>`
                    }
                    
                    <p class="wiz-question-answer" style="line-height: 1; margin: 0; padding: 0;"><strong style="font-weight: 600; margin-right: 4px;">Correct Answer(s):</strong></p>${element.answer}<p></p>
                    <p class="explanation-header" style="font-weight: 600; margin-bottom: 4px;">Explanation:</p>
                    <div class="wiz-question-explanation" style="font-size: 13px; line-height: 1.4; margin-bottom: 8px;">${element.explanation}</div>
                    <p class="disclaimer" style="font-size: 12px; color: #666; font-style: italic; margin-bottom: 12px; line-height: 1.1;">The explanation provided above is AI-generated and may not be entirely accurate. Please verify with trusted sources.</p>
                    <div class="watermark-container" style="display: flex; justify-content: flex-end;">
                        <img src="chrome-extension://kliaonmakfpkhojpcicfahbanaeifjoo/img/watermark.png" alt="Watermark by CanvasWizard.io" style="height: 24px; width: auto;">
                    </div>
                </div>
            </div>
            `;
        });
    });
}
loadHistory();

// Listen to changes made to the wiz_history key in the local storage, and update the chat history accordingly
chrome.storage.onChanged.addListener(function (changes) {
    if (changes.wiz_history) {
        loadHistory();
    }
});

document.querySelector('[wizdata-button="history"]').addEventListener('click', () => {
    chrome.storage.local.get(['wiz_settings']).then(result => {
        chrome.storage.local.set({ 
            wiz_settings: {
                ...result.wiz_settings,
                history_visible: !result.wiz_settings.history_visible,
                chat_visible: false
            }
        });
    });
});

document.querySelector('[wizdata-button="clear_wizhistory"]').addEventListener('click', function() {
    chrome.storage.local.set({ wiz_history: [] }, function() {
        loadHistory();
    });
});
