// const API_ENDPOINT = "http://localhost:9000";
const API_ENDPOINT = "https://api.canvaswizard.co";

// WIZ_SETTINGS
chrome.storage.local.get(['wiz_settings'], (result) => {
    const template_settings = {
        incognito_mode: false,
        sidebar_visible: true,
        screenshot_visible: false,
        chat_visible: false,
        history_visible: false,
        extension_uuid: crypto.randomUUID(),
        is_subscribed: false,
    };
    if (!result.wiz_settings) {
        // Set default settings
        chrome.storage.local.set({ wiz_settings: template_settings });
    } else {
        // Set body attributes from existing settings, falling back to defaults
        const settings = result.wiz_settings;
        document.body.setAttribute('wizdata-incognito', settings.incognito_mode ?? template_settings.incognito_mode);
        document.body.setAttribute('wizdata-sidebar', settings.sidebar_visible ?? template_settings.sidebar_visible);
        document.body.setAttribute('wizdata-screenshot', settings.screenshot_visible ?? template_settings.screenshot_visible);
        document.body.setAttribute('wizdata-chat', settings.chat_visible ?? template_settings.chat_visible);
        document.body.setAttribute('wizdata-history', settings.history_visible ?? template_settings.history_visible);  
    }
});

// WIZ_HISTORY
chrome.storage.local.get(['wiz_history'], (result) => {
    if (!result.wiz_history) {
        // Set an empty list that will be filled afterwards.
        chrome.storage.local.set({ wiz_history: [] });
    }
});

// WIZ_NOTES
chrome.storage.local.get(['wiz_notes'], (result) => {
    if (!result.wiz_notes) {
        // Set an empty list that will be filled afterwards.
        chrome.storage.local.set({ wiz_notes: [] });
    }
});

// WIZ_CHAT
chrome.storage.local.get(['wiz_chat'], (result) => {
    if (!result.wiz_chat) {
        // Set an empty list that will be filled afterwards.
        chrome.storage.local.set({ wiz_chat: [{ role: 'assistant', content: 'Hi! I\'m WizChat, I\'m here help your to answer any of your questions!' }] });
    }
});

// LOCAL STORAGE CHANGES LISTENER
chrome.storage.onChanged.addListener((changes, namespace) => {
    if (namespace === 'local' && changes.wiz_settings) {
        const new_settings = changes.wiz_settings.newValue;
        // Update body attributes when settings change
        if (new_settings) {
            if (new_settings.hasOwnProperty('incognito_mode')) {
                document.body.setAttribute('wizdata-incognito', new_settings.incognito_mode);
            }
            if (new_settings.hasOwnProperty('sidebar_visible')) {
                document.body.setAttribute('wizdata-sidebar', new_settings.sidebar_visible);
            }
            if (new_settings.hasOwnProperty('screenshot_visible')) {
                document.body.setAttribute('wizdata-screenshot', new_settings.screenshot_visible); 
            }
            if (new_settings.hasOwnProperty('chat_visible')) {
                document.body.setAttribute('wizdata-chat', new_settings.chat_visible);
            }
            if (new_settings.hasOwnProperty('history_visible')) {
                document.body.setAttribute('wizdata-history', new_settings.history_visible);
            }
        }
    }
});

// FILL THE INPUT
function populateExtensionUuid() {
    if (!window.location.href.startsWith('https://canvaswizard.co')) {return;}
    const extension_uuid_field = document.querySelector('input[name="wizard_extension_uuid"]');
    if (!extension_uuid_field) {return;}
    chrome.storage.local.get(['wiz_settings'], result => {
        const uuid = result.wiz_settings?.extension_uuid;
        if (uuid) { extension_uuid_field.value = uuid; }
    });
}
populateExtensionUuid();

// PRINT LOCAL STORAGE VALUES
// chrome.storage.local.get(null, (items) => {
//     console.group('Chrome Local Storage Contents:');
//     Object.entries(items).forEach(([key, value]) => {
//         console.log(`${key}:`, value);
//     });
//     console.groupEnd();
// });
