// Function to generate post its
function createPostItElement(content = '', noteId = Math.random().toString(36).substring(2, 15), position, title="Note") {
    const postit = document.createElement('div');
    postit.className = 'wiz-style wiz-postit wiz-ui';
    postit.style.borderRadius = '4px';
    postit.style.zIndex = '99999';
    postit.style.position = 'absolute';

    // Check if position is provided and valid, otherwise center it
    if (position && typeof position === 'object' && position.left !== undefined && position.top !== undefined) {
        postit.style.left = position.left; // Use custom left position
        postit.style.top = position.top; // Use custom top position
    } else {
        // Center the post-it note in the viewport
        postit.style.left = `${(window.innerWidth / 2) - (postit.offsetWidth / 2)}px`; // Center horizontally
        postit.style.top = `${(window.innerHeight / 2 + window.scrollY) - (postit.offsetHeight / 2)}px`; // Center vertically considering scroll
    }

    postit.style.transform = 'translate(-50%, -50%)';
    postit.setAttribute('data-wiz-note-id', noteId);
    
    postit.innerHTML = `
        <div class="wiz-postit-dragbar" style="
            background-color: #1A1F22;
            border: 3px solid #00E6D2;
            border-radius: 4px;
            width: 256px;
            height: 256px;
            display: flex;
            flex-direction: column;
        ">
            <div style="
                height: 1.25rem;
                width: 100%;
                background-color: #00E6D2;
                cursor: move;
                display: flex;
                justify-content: space-between;
                align-items: center;
                padding: 0 0.5rem;
            ">
                <span style="
                    font-size: 0.875rem;
                    color: #1A1F22;
                    font-weight: bold;
                    text-transform: capitalize;
                ">${title}</span>
                <div style="display: flex; align-items: center;">
                    <button style="
                        color: #1A1F22;
                        font-size: 1.25rem;
                        transition: color 0.3s;
                    ">&times;</button>
                </div>
            </div>
            <textarea style="
                flex: 1;
                border: none;
                outline: none;
                width: 100%;
                padding: 0.5rem;
                background-color: transparent;
                color: white;
                resize: none;
            ">${content}</textarea>
        </div>
    `;
    
    document.body.appendChild(postit);

    const dragBar = postit.querySelector('.wiz-postit-dragbar');
    const textarea = postit.querySelector('textarea');
    let isDragging = false;
    let startX, startY, startLeft, startTop;

    function saveNoteData() {
        chrome.storage.local.get(['wiz_notes'], function(result) {
            let notes = result.wiz_notes || [];
            
            // Find existing note index
            const noteIndex = notes.findIndex(note => note.id === noteId);
            
            const noteData = {
                id: noteId,
                content: textarea.value,
                position: {
                    left: postit.style.left,
                    top: postit.style.top
                }
            };

            // Update existing note or add new one
            if (noteIndex !== -1) {
                notes[noteIndex] = noteData;
            } else {
                notes.push(noteData);
            }

            // Save updated notes array
            chrome.storage.local.set({wiz_notes: notes}, function() {
                if (chrome.runtime.lastError) {
                    console.error('Error saving note:', chrome.runtime.lastError);
                }
            });
        });
    }

    dragBar.addEventListener('mousedown', function(e) {
        isDragging = true;
        startX = e.clientX;
        startY = e.clientY;
        startLeft = postit.offsetLeft;
        startTop = postit.offsetTop;
    });

    document.addEventListener('mousemove', function(e) {
        if (!isDragging) return;
        const dx = e.clientX - startX;
        const dy = e.clientY - startY;
        postit.style.left = startLeft + dx + 'px';
        postit.style.top = startTop + dy + 'px';
        saveNoteData();
    });

    document.addEventListener('mouseup', function() {
        isDragging = false;
    });

    postit.querySelector('button').addEventListener('click', function() {
        document.body.removeChild(postit);
        chrome.storage.local.get(['wiz_notes'], function(result) {
            let notes = result.wiz_notes || [];
            notes = notes.filter(note => note.id !== noteId);
            chrome.storage.local.set({wiz_notes: notes}, function() {
                if (chrome.runtime.lastError) {
                    console.error('Error removing note:', chrome.runtime.lastError);
                }
            });
        });
    });

    textarea.addEventListener('input', saveNoteData);
    return postit;
}

// NOTES
document.querySelector('[wizdata-button="notes"]').addEventListener('click', function() {
    createPostItElement();
});

// Retrieve and display existing notes.
function displayStoredNotes() {
    chrome.storage.local.get(['wiz_notes'], function(result) {
        if (chrome.runtime.lastError) {
            console.error('Error retrieving notes:', chrome.runtime.lastError);
            return;
        }
        
        const notes = result.wiz_notes || [];
        notes.forEach(note => {
            if (note.content && note.id && note.position) {
                createPostItElement(note.content, note.id, note.position);
            }
        });
    });
}
displayStoredNotes();



