const API_ENDPOINT = "https://api.canvaswizard.co";

function displaySubscribedScreen(is_subscribed) {
    const elements = {
        screens: {
            need_activation: document.getElementById('need_activation_screen'),
            enabled: document.getElementById('enabled_screen'),
            disabled: document.getElementById('disabled_screen'),
            loading: document.getElementById('loading_screen')
        },
        status: {
            bg_elipse: document.getElementById('background_elipse'),
            indicator: document.getElementById('status_indicator'),
            enabled: document.getElementById('status_enabled'),
            disabled: document.getElementById('status_disabled')
        }
    };

    // Hide all screens
    Object.values(elements.screens).forEach(screen => screen.classList.add('hidden'));

    // Update UI based on subscription status
    const activeScreen = is_subscribed ? 'enabled' : 'need_activation';
    elements.screens[activeScreen].classList.remove('hidden');
    elements.status.bg_elipse.setAttribute('data-active', String(is_subscribed));
    elements.status.indicator.innerText = is_subscribed ? 'Enabled' : 'Disabled';
    elements.status.enabled.classList.toggle('hidden', !is_subscribed);
    elements.status.disabled.classList.toggle('hidden', is_subscribed);
}

function checkAuthentication() {
    chrome.storage.local.get(['wiz_settings'], ({ wiz_settings = {} }) => {
        const { is_subscribed = false, extension_uuid = false } = wiz_settings;
        displaySubscribedScreen(true);  // Always treat as subscribed
        document.getElementById("extension_uuid").innerText = extension_uuid;

        // Skip the actual API call and just assume the user is subscribed
        const newSettings = {
            ...wiz_settings,
            is_subscribed: true // Always mark as subscribed
        };
        chrome.storage.local.set({ wiz_settings: newSettings }, () => {
            displaySubscribedScreen(true); // Always show enabled screen
        });
    });
}

// On incognito_mode_trigger click, toggle incognito storage data, then update the button text
document.getElementById('incognito_mode_trigger').addEventListener('click', () => {
    chrome.storage.local.get(['wiz_settings'], ({ wiz_settings = {} }) => {
        const newSettings = { ...wiz_settings, incognito_mode: !wiz_settings.incognito_mode };
        chrome.storage.local.set({ wiz_settings: newSettings });
        console.log("Incognito mode: " + newSettings.incognito_mode);
    });
});

// Initialize button text based on current incognito mode state
chrome.storage.local.get(['wiz_settings'], ({ wiz_settings = {} }) => {
    document.getElementById('incognito_mode_trigger').innerText = wiz_settings.incognito_mode ? 'Disable Incognito Mode' : 'Enable Incognito Mode';
});

chrome.storage.onChanged.addListener((changes, namespace) => {
    if (namespace === 'local' && changes.wiz_settings) {
        const newSettings = changes.wiz_settings.newValue;
        document.getElementById('incognito_mode_trigger').innerText = 
            newSettings.incognito_mode ? 'Disable Incognito Mode' : 'Enable Incognito Mode';
    }
});

checkAuthentication();

// If somebody press the check_activation button in the popup, we will check the authentication status
document.getElementById('check_activation').addEventListener('click', checkAuthentication);
// document.getElementById('update_billing').addEventListener('click', () => chrome.tabs.create({ url: 'https://canvaswizard.co/bill' }));
// document.getElementById('feedback_or_contact').addEventListener('click', () => chrome.tabs.create({ url: 'mailto:hi@canvaswizard.co' }));

document.getElementById('feedback_or_contact').addEventListener('click', () => {
    chrome.storage.local.get(['wiz_settings'], ({ wiz_settings = {} }) => {
        const extension_uuid = wiz_settings.extension_uuid || '';
        const bodyText = '\n\n\n\n\n---\nPlease avoid deleting this, as it\'s essential for assisting you.\nExtension ID: ' + extension_uuid;

        chrome.tabs.create({ url: 'mailto:hi@canvaswizard.co?body=' + encodeURIComponent(bodyText) });
    });
});

document.getElementById('activate_extension_button').addEventListener('click', () => {
    chrome.storage.local.get(['wiz_settings'], ({ wiz_settings = {} }) => {
        const extension_uuid = wiz_settings.extension_uuid || '';
        chrome.tabs.create({ url: 'https://canvaswizard.co/api/extension/activation?extension_uuid=' + extension_uuid });
    });
});
