// SCREENSHOT OVERLAY
const wiz_screenshot_overlay = document.createElement('div');
wiz_screenshot_overlay.id = "wiz-screenshot-overlay"; 
wiz_screenshot_overlay.className = "wiz-style wiz-ui"; // Changed 'relative' to 'fixed' to keep it always on top
wiz_screenshot_overlay.style.cssText = `background-color: #00E6D24D; display: none;  position: fixed;  top: 0;  left: 0;  justify-content: flex-start;  align-items: flex-end;  width: 100vw;  height: 100vh; z-index: 99999; `
wiz_screenshot_overlay.innerHTML = `
    <div style="position: absolute; top: 16px; right: 104px;">
        <button wizdata-button="screenshot_overlay_close" style="display: flex; justify-content: center; align-items: center; transition-duration: 200ms; background: #1A2426; border-color: #125D59; border-width: 2px; border-radius: 4px; padding: 6px; color: #00E6D2; :hover {color: white;}">
            <svg xmlns="http://www.w3.org/2000/svg" style="width: 20px;  height: 20px;" viewBox="0 0 24 24"><path fill="currentColor" d="m12 13.4l-4.9 4.9q-.275.275-.7.275t-.7-.275t-.275-.7t.275-.7l4.9-4.9l-4.9-4.9q-.275-.275-.275-.7t.275-.7t.7-.275t.7.275l4.9 4.9l4.9-4.9q.275-.275.7-.275t.7.275t.275.7t-.275.7L13.4 12l4.9 4.9q.275.275.275.7t-.275.7t-.7.275t-.7-.275z"/></svg>
        </button>
    </div>
    <div id="wiz-screenshot-explanation" style="display: flex;  padding: 16px;  margin-top: 8px;  flex-direction: column;  justify-content: center;  align-items: center;  pointer-events: none;  user-select: none;  width: 300px;">
        <div style="aspect-ratio: 1 / 1; padding: 24px; border-radius: 4px; width: 100%; height: auto; border: 2px solid #00E6D2; background-color: #1A2426;">
            <div style="display: flex;  padding: 16px;  justify-content: center;  align-items: center;  border-radius: 2px;  height: 100%;  background-color: #F9FAFB;">
                <img style="width: 100%;  height: 100%;  user-select: none;" src="${chrome.runtime.getURL('img/screenshot-question.gif')}" alt="Screenshot explanation image, showing how a user selects an area.">
            </div>
        </div>
        <div style="background-color: #1A2426; border: 2px solid #00E6D2; padding: 12px; border-radius: 4px; margin-top: 8px;">
            <p style="line-height: 1.25; font-size: 14px; text-align: left; letter-spacing: -0.4px; margin: 0; color: white; ">Click and drag the area you want to screenshot.</p>
        </div>
    </div>
`;

document.body.appendChild(wiz_screenshot_overlay);

// SHOW SCREENSHOT OVERLAY
document.querySelector('[wizdata-button="screenshot"]').addEventListener('click', () => {
    chrome.storage.local.get(['wiz_settings']).then(result => {
        chrome.storage.local.set({ 
            wiz_settings: {
                ...result.wiz_settings,
                screenshot_visible: true,
                chat_visible: false,
                history_visible: false,
            }
        });
    });
});

// ESC KEY (Close the overlay)
document.addEventListener('keydown', (event) => {
    if (event.key === 'Escape') {
        chrome.storage.local.get(['wiz_settings']).then(result => {
            if (result.wiz_settings.screenshot_visible) {
                chrome.storage.local.set({
                    wiz_settings: {
                        ...result.wiz_settings,
                        screenshot_visible: false
                    }
                });
            }
        });
    }
});

// Setup popup interaction
function setupPopupInteraction(popup) {
    const dragbar = popup.querySelector('.wiz-dragbar');
    const closeBtn = popup.querySelector('button');
    let isDragging = false, startX, startY, startLeft, startTop;

    dragbar.addEventListener('mousedown', e => {
        isDragging = true;
        startX = e.clientX;
        startY = e.clientY;
        startLeft = popup.offsetLeft;
        startTop = popup.offsetTop;
    });

    dragbar.addEventListener('mousemove', e => {
        if (!isDragging) return;
        const dx = e.clientX - startX;
        const dy = e.clientY - startY;
        popup.style.left = startLeft + dx + 'px';
        popup.style.top = startTop + dy + 'px';
    });

    dragbar.addEventListener('mouseup', () => isDragging = false);
    closeBtn.addEventListener('click', () => popup.remove());
}

// RETRIEVE ANSWER
function getAnswerFromScreenshot(cropped_image_base64, start_x, end_x, start_y, end_y){
    // Display answer in center of selected area
    // Create wiz_screenshot_answer_popup container
    // Calculate center position of selected area
    const center_x = Math.min(start_x, end_x) + Math.abs(end_x - start_x) / 2;
    const center_y = Math.min(start_y, end_y) + Math.abs(end_y - start_y) / 2;

    const wiz_screenshot_answer_popup = document.createElement('div');
    wiz_screenshot_answer_popup.className = 'wiz-ui wiz-style';
    wiz_screenshot_answer_popup.style.cssText = `
        position: absolute;
        z-index: 999999;
        left: ${center_x}px;
        top: ${center_y}px;
        transform: translate(-50%, -50%);
        background-color: transparent;
    `;

    // Set wiz_screenshot_answer_popup content
    wiz_screenshot_answer_popup.innerHTML = `
        <div style="background-color: #1E1E1E; border: 3px solid #00E6D2; border-radius: 4px; width: 200px; height: auto; display: flex; flex-direction: column;">
            <div class="wiz-dragbar" style="height: 20px; width: 100%; background-color: #00E6D2; cursor: move; display: flex; justify-content: space-between; align-items: center; padding: 2px 2px;">
                <span style="font-size: 14px; pointer-events: none; color: #1E1E1E; text-transform: capitalize; font-weight: bold;">Screenshot Answer</span>
                <div style="display: flex; align-items: center;">
                    <button style="color: #1E1E1E; font-size: 20px; transition: color 300ms;">&times;</button>
                </div>
            </div>
            <div style="pointer-events: none; display: flex; justify-content: center; align-items: center; height: 100%; background-color: #f5f5f5; height: 210px; width: 100%; position: relative;">
                <div class="spinner" style="border: 4px solid rgba(0, 0, 0, 0.1); border-top: 4px solid #00E6D2; border-radius: 50%; width: 30px; height: 30px; animation: spin 1s linear infinite;"></div>
                <span style="margin-left: 10px; font-size: 16px; color: #1E1E1E;">Loading...</span>
                <div class="watermark-container" style="position: absolute; bottom: 4px; right: 4px;">
                     <img src=${chrome.runtime.getURL('img/watermark.png')} alt="Watermark by CanvasWizard.io" style="height: 24px; width: auto;">
                </div>
            </div>
            <style>
                @keyframes spin {
                    0% { transform: rotate(0deg); }
                    100% { transform: rotate(360deg); }
                }
            </style>
        </div>
    `;

    const wiz_screenshot_answer_dragbar = wiz_screenshot_answer_popup.querySelector('.wiz-dragbar');
    let isDragging = false;

    wiz_screenshot_answer_dragbar.addEventListener('mousedown', function(e) {
        isDragging = true;
        startX = e.clientX;
        startY = e.clientY;
        startLeft = wiz_screenshot_answer_popup.offsetLeft;
        startTop = wiz_screenshot_answer_popup.offsetTop;
    });

    wiz_screenshot_answer_dragbar.addEventListener('mousemove', function(e) {
        if (!isDragging) return;
        const dx = e.clientX - startX;
        const dy = e.clientY - startY;
        wiz_screenshot_answer_popup.style.left = startLeft + dx + 'px';
        wiz_screenshot_answer_popup.style.top = startTop + dy + 'px';
    });

    wiz_screenshot_answer_dragbar.addEventListener('mouseup', function() {
        isDragging = false;
    });

    // Close popup
    wiz_screenshot_answer_popup.querySelector('button').addEventListener('click', () => wiz_screenshot_answer_popup.remove());   

    // Add to document
    document.body.appendChild(wiz_screenshot_answer_popup);

    chrome.storage.local.get('wiz_settings', function(result) {
        extension_uuid = result.wiz_settings?.extension_uuid || '';

        fetch(`${API_ENDPOINT}/screenshot`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'Extension-UUID': extension_uuid
            },
            body: JSON.stringify({
                question_base64: cropped_image_base64,
            })
        })
        .then(response => {
            if (!response.ok) {throw new Error(response.status);}
            return response.json();
        })
        .then(data => {
            // Get existing history and append new entry
            chrome.storage.local.get(['wiz_history'], (result) => {
                const history = result.wiz_history || [];
                history.unshift({
                    uuid: data.data.uuid,
                    question: cropped_image_base64,
                    answer: data.data.answer,
                    explanation: data.data.explanation,
                });
                // Store updated history
                chrome.storage.local.set({ wiz_history: history });
            });

            // Set wiz_screenshot_answer_popup content
            wiz_screenshot_answer_popup.innerHTML = `
                <div style="background-color: #1E1E1E; border: 3px solid #00E6D2; border-radius: 4px; width: 350px; height: auto; display: flex; flex-direction: column;">
                    <div class="wiz-dragbar" style="height: 20px; width: 100%; background-color: #00E6D2; cursor: move; display: flex; justify-content: space-between; align-items: center; padding: 2px 2px;">
                        <span style="font-size: 14px; pointer-events: none; color: #1E1E1E; text-transform: capitalize; font-weight: bold;">Screenshot Answer</span>
                        <div style="display: flex; align-items: center;">
                            <button style="color: #1E1E1E; font-size: 20px; transition: color 300ms;">&times;</button>
                        </div>
                    </div>
                    <div style="pointer-events: none; color: #2c3b44; background-color: #f5f5f5; font-weight: normal; padding: 8px; font-size: 14px;">
                        <p class="wiz-question-answer" style="line-height: 1; margin: 0; padding: 0;"><strong style="font-weight: 600; margin-right: 4px;">Correct Answer(s):</strong> ${data.data.answer}</p>
                        <p class="explanation-header" style="font-weight: 600; margin-bottom: 4px;">Explanation:</p>
                        <div class="wiz-question-explanation" style="font-size: 13px; line-height: 1.4; margin-bottom: 8px;">${data.data.explanation}</div>
                        <p class="disclaimer" style="font-size: 12px; color: #666; font-style: italic; margin-bottom: 12px; line-height: 1.1;">The explanation provided above is AI-generated and may not be entirely accurate. Please verify with trusted sources.</p>
                        <div class="watermark-container" style="display: flex; justify-content: flex-end;">
                            <img src=${chrome.runtime.getURL('img/watermark.png')} alt="Watermark by CanvasWizard.io" style="height: 24px; width: auto;">
                        </div>
                    </div>
                </div>
            `;

            setupPopupInteraction(wiz_screenshot_answer_popup);
        })
        .catch(error => {
            if (error instanceof Error && error.message) {
                const statusCode = parseInt(error.message.match(/\d+/)[0]);
                if (statusCode === 401) {
                    error_message = "Your account is not activated. Please activate it by visiting the setup page or clicking the extension icon.";
                } else {
                    error_message = "An error occurred while processing this question. Please try again.";
                }
            } else {
                error_message = "An unexpected error occurred. Please try again.";
            }

            wiz_screenshot_answer_popup.innerHTML = `
                <div style="background-color: #1E1E1E; border: 3px solid #00E6D2; border-radius: 4px; width: 350px; height: auto; display: flex; flex-direction: column;">
                    <div class="wiz-dragbar" style="height: 20px; width: 100%; background-color: #00E6D2; cursor: move; display: flex; justify-content: space-between; align-items: center; padding: 2px 2px;">
                        <span style="font-size: 14px; pointer-events: none; color: #1E1E1E; text-transform: capitalize; font-weight: bold;">Screenshot Answer</span>
                        <div style="display: flex; align-items: center;">
                            <button style="color: #1E1E1E; font-size: 20px; transition: color 300ms;">&times;</button>
                        </div>
                    </div>
                    <div style="pointer-events: none; display: flex; justify-content: center; align-items: center; height: 100%; background-color: #f5f5f5; height: 120px; width: 100%; position: relative;">
                        <span style="margin-left: 10px; font-size: 13.6px; color: #1E1E1E;">${error_message}</span>
                        <div class="watermark-container" style="position: absolute; bottom: 4px; right: 4px;">
                            <img src=${chrome.runtime.getURL('img/watermark.png')} alt="Watermark by CanvasWizard.io" style="height: 24px; width: auto;">
                        </div>
                    </div>
                </div>
            `;

            setupPopupInteraction(wiz_screenshot_answer_popup);
        });
    });
};

// SCREENSHOT SELECTION
document.querySelector('#wiz-screenshot-overlay').addEventListener('mousedown', function handleScreenshotMouseDown(event) {
    // Prevent default browser behavior
    event.preventDefault();

    // Change the localstoage to screenshot turned off
    chrome.storage.local.get(['wiz_settings']).then(result => {
        chrome.storage.local.set({ 
            wiz_settings: {
                ...result.wiz_settings,
                screenshot_visible: false,
            }
        });
    });

    // Set cursor and prevent text selection
    document.body.style.cursor = 'crosshair';
    document.body.style.userSelect = 'none';

    // Get initial mouse position
    const start_x = event.clientX;
    const start_y = event.clientY + window.scrollY;

    // Create selection box element
    const wiz_selection_box = document.createElement('div');
    wiz_selection_box.classList.add('wiz-style', 'wiz-ui');
    wiz_selection_box.style.cssText = `
        position: fixed;
        border: 2px dashed #00E6D2;
        background: rgba(0, 230, 210, 0.1);
        z-index: 999999;
        border-radius: 4px;
    `;

    // Add wiz_screenshot_tooltip label
    const wiz_screenshot_tooltip = document.createElement('div');
    wiz_screenshot_tooltip.style.cssText = `
        position: absolute;
        top: 0;
        right: 0;
        background: #00E6D2;
        color: #1A2426;
        padding: 4px 8px;
        border-bottom-left-radius: 4px;
        font-size: 14px;
        white-space: nowrap;
    `;
    wiz_screenshot_tooltip.textContent = 'Screenshot Area';
    wiz_selection_box.appendChild(wiz_screenshot_tooltip);
    document.body.appendChild(wiz_selection_box);

    // Handle mouse move
    function handleMouseMove(e) {
        const current_x = e.clientX;
        const current_y = e.clientY + window.scrollY;

        // Calculate dimensions accounting for scroll
        const width = Math.abs(current_x - start_x);
        const height = Math.abs(current_y - start_y);
        const left = Math.min(current_x, start_x);
        const top = Math.min(current_y - window.scrollY, start_y - window.scrollY);

        // Update selection box
        wiz_selection_box.style.width = `${width}px`;
        wiz_selection_box.style.height = `${height}px`; 
        wiz_selection_box.style.left = `${left}px`;
        wiz_selection_box.style.top = `${top}px`;
    }

    // Handle mouse up
    function handleMouseUp(e) { 
        // Clean up event listeners
        document.removeEventListener('mousemove', handleMouseMove);
        document.removeEventListener('mouseup', handleMouseUp);
        // Mouse cursor position
        const end_x = e.clientX;
        const end_y = e.clientY + window.scrollY;
        // Reset cursor and text selection
        document.body.style.cursor = 'default';
        document.body.style.userSelect = 'text';
        wiz_selection_box.remove();
        // Remove selection box after slight delay
        setTimeout(() => {
            chrome.runtime.sendMessage({ msg: "capture" }, (response) => {
                if (chrome.runtime.lastError) {
                    console.error("Error capturing screenshot:", chrome.runtime.lastError.message);
                    return;
                }
                const canvas = document.createElement('canvas');
                const context = canvas.getContext('2d');
                const image = new Image();
                image.src = response.screenshot;

    
                image.onload = () => {
                    const browser_zoom_level = window.devicePixelRatio || 1;
                    const width = Math.abs(end_x - start_x);
                    const height = Math.abs(end_y - start_y);
                    // Missclick or screenshot size to small.
                    if (width < 50 && height < 50) {return;}
                    // Cropping the image
                    canvas.width = width;
                    canvas.height = height;
                    context.drawImage(image, 
                        Math.min(start_x, end_x) * browser_zoom_level, 
                        Math.min(start_y - window.scrollY, end_y - window.scrollY) * browser_zoom_level, 
                        canvas.width * browser_zoom_level,
                        canvas.height * browser_zoom_level,
                        0, 0, canvas.width, canvas.height
                    );
                    const cropped_image_base64 = canvas.toDataURL();
                    // Get the answer
                    getAnswerFromScreenshot(cropped_image_base64, start_x, end_x, start_y, end_y);
                };
            });
        }, 200);
    }

    // Add event listeners
    document.addEventListener('mousemove', handleMouseMove);
    document.addEventListener('mouseup', handleMouseUp);
});


