// Function to create and position the wiz_writing_popup
function createWritingPopup(range) {
    const rect = range.getBoundingClientRect();
    const scrollX = window.scrollX || window.pageXOffset;
    const scrollY = window.scrollY || window.pageYOffset;
    const wiz_writing_popup = document.createElement('div');
    wiz_writing_popup.id = 'wiz-writing-popup';
    wiz_writing_popup.className = 'bg-wiz-darkgreen border-wiz-mint wiz-ui wiz-style';
    wiz_writing_popup.style.cssText = `
        display: flex;
        position: absolute;
        padding: 6px;
        margin-left: 0px;
        align-items: center;
        border-radius: 9999px;
        box-shadow: 0 10px 15px -3px rgba(0, 0, 0, 0.1), 0 4px 6px -2px rgba(0, 0, 0, 0.05);
        border-width: 1.5px !important;
        z-index: 9999;
        left: ${rect.left + scrollX}px;
        top: ${rect.bottom + scrollY +  10}px;
    `;
    
    const writing_options_buttons = `
        <div id="wiz-writing-options-buttons" style="display: inline-flex; flex-grow: 1; align-items: center; border-radius: 9999px;">
            <button class="bg-wiz-mint hover:text-wiz-mint" style="display: flex;  padding-top: 6px; padding-bottom: 6px;  padding-left: 16px; padding-right: 16px;  flex-grow: 1;  justify-content: center;  align-items: center;  border-top-left-radius: 9999px; border-bottom-left-radius: 9999px;  font-size: 12px; line-height: 16px;  font-weight: 600;  transition-property: color, background-color, border-color, text-decoration-color, fill, stroke; transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1); transition-duration: 300ms; transition-duration: 200ms; color: #1e293b;  :hover {  background: #152D2E;  }" wizdata-action="summarize" wizdata-title="Summarized Text">
               <svg xmlns="http://www.w3.org/2000/svg" style="width: 16px;  height: 16px; margin-right: 4px;" viewBox="0 0 24 24"><path fill="currentColor" d="M4 5h16v2H4zm0 4h16v2H4zm0 4h16v2H4zm0 4h10v2H4z"/></svg>
                <span>Summarize</span>
            </button>
            <button class="bg-wiz-mint hover:text-wiz-mint" style="display: flex;  padding-top: 6px; padding-bottom: 6px;  padding-left: 16px; padding-right: 16px;  margin-left: 4px;  flex-grow: 1;  justify-content: center;  align-items: center;  border-top-right-radius: 9999px; border-bottom-right-radius: 9999px;  font-size: 12px; line-height: 16px;  font-weight: 600;  transition-property: color, background-color, border-color, text-decoration-color, fill, stroke; transition-timing-function: cubic-bezier(0.4, 0, 0.2, 1); transition-duration: 300ms; transition-duration: 200ms; color: #1e293b;  :hover {  background: #152D2E;  }"  wizdata-action="rewrite" wizdata-title="Rewritten Text">
                <svg xmlns="http://www.w3.org/2000/svg" style="width: 12px;  height: 12px; margin-right: 4px;" viewBox="0 0 14 14"><path fill="currentColor" fill-rule="evenodd" d="M10.213 2.538A5.499 5.499 0 0 0 1.595 8.01a.75.75 0 0 1-1.474.277a6.999 6.999 0 0 1 11.163-6.821l.612-.612a.5.5 0 0 1 .854.353V3.5a.5.5 0 0 1-.5.5H9.957a.5.5 0 0 1-.353-.853zm2.791 2.577a.75.75 0 0 1 .876.598a6.999 6.999 0 0 1-11.164 6.821l-.612.613a.5.5 0 0 1-.854-.354V10.5a.5.5 0 0 1 .5-.5h2.293a.5.5 0 0 1 .354.854l-.61.609a5.499 5.499 0 0 0 8.618-5.472a.75.75 0 0 1 .6-.876ZM6.181 4.034c.19-.87 1.427-.874 1.625-.008l.01.044a2.69 2.69 0 0 0 2.18 2.123c.904.157.904 1.457-.001 1.614a2.69 2.69 0 0 0-2.164 2.054l-.025.113c-.198.867-1.435.861-1.625-.007L6.16 9.87a2.68 2.68 0 0 0-2.156-2.064c-.903-.157-.903-1.454 0-1.611A2.68 2.68 0 0 0 6.157 4.14l.016-.071z" clip-rule="evenodd"/></svg>
                <span>Rewrite</span>
            </button>
        </div>
    `;
    
    wiz_writing_popup.insertAdjacentHTML('beforeend', writing_options_buttons);
    
    const user_selected_text = window.getSelection().toString();
    const action_buttons = wiz_writing_popup.querySelectorAll('button');
    action_buttons.forEach(btn => {
        btn.addEventListener('click', (e) => {
            e.stopPropagation();
            handleWritingButtonClick(btn.getAttribute('wizdata-action'), user_selected_text);

            // Remove existing content from wiz_toolbar
            const wiz_writing_options_buttons = document.getElementById('wiz-writing-options-buttons');
            wiz_writing_options_buttons.innerHTML = '';

            // Add loading indicator
            const loading_spinner = document.createElement('div');
            loading_spinner.style.cssText = 'display: flex; justify-content: center; align-items: center; width: 100%;';
            loading_spinner.innerHTML = `
                <svg style="margin-left: 8px;  width: 16px;  height: 16px;" class="animate-spin  text-wiz-mint" xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 24">
                    <circle style="opacity: 0.25;" cx="12" cy="12" r="10" stroke="currentColor" stroke-width="4"></circle>
                    <path style="opacity: 0.75;" fill="currentColor" d="M4 12a8 8 0 018-8V0C5.373 0 0 5.373 0 12h4zm2 5.291A7.962 7.962 0 014 12H0c0 3.042 1.135 5.824 3 7.938l3-2.647z"></path>
                </svg>
                <span style="margin-left: 8px; font-size: 12px; line-height: 16px;" class="text-wiz-mint font-semibold">Loading...</span>
            `;
            wiz_writing_options_buttons.appendChild(loading_spinner);

            // Generate a post request to the API with the selection and the option
            const data = {
                selection: user_selected_text,
                action: btn.getAttribute('wizdata-action')
            };

            chrome.storage.local.get('wiz_settings', function(result) {
                extension_uuid = result.wiz_settings?.extension_uuid || '';

                fetch(`${API_ENDPOINT}/selection`, {
                    method: 'POST',
                    headers: {
                        'Content-Type': 'application/json',
                        'Extension-UUID': extension_uuid,
                    },
                    body: JSON.stringify(data)
                })
                .then(response => response.json())
                .then(result => {
                    wiz_writing_popup.remove();
                    createPostItElement(result.data, undefined, undefined, "Writing Tools");
                })
                .catch(error => {
                    console.error('Error:', error);

                    // Handle any errors here
                    wiz_writing_options_buttons.innerHTML = `
                    <svg xmlns="http://www.w3.org/2000/svg" style="margin-left: 8px;  width: 16px;  height: 16px;" class="text-wiz-mint" viewBox="0 0 24 24"><path fill="currentColor" d="M12 2c5.523 0 10 4.478 10 10s-4.477 10-10 10S2 17.522 2 12S6.477 2 12 2m.002 13.004a.999.999 0 1 0 0 1.997a.999.999 0 0 0 0-1.997M12 7a1 1 0 0 0-.993.884L11 8l.002 5.001l.007.117a1 1 0 0 0 1.986 0l.007-.117L13 8l-.007-.117A1 1 0 0 0 12 7"/></svg>
                    <span class="text-wiz-mint font-semibold" style="font-size: 12px; line-height: 16px; margin-left: 8px;">An error occurred, try again later.</span>
                    `;
                });
            });

        });
    });
    
    wiz_writing_popup.insertAdjacentHTML('beforeend', `
        <div class="border-wiz-mint" style="padding: 6px; margin-left: 16px; border-radius: 9999px; background: #1B3536; border: 1.5px solid #00E6D2; height:28px; width:28px;">
            <img src="${chrome.runtime.getURL('img/logo.webp')}" style="width: auto; height: 100%; margin-bottom: 1px; margin-left: 1px;">
        </div>
    `);
    document.body.appendChild(wiz_writing_popup);
    adjustWritingPopupPosition();
}

// Function to handle button clicks
function handleWritingButtonClick(action, user_selected_text) {
    console.log(`${action}ing:`, user_selected_text);
    // Add action-specific logic here
}

// Function to remove the wiz_writing_popup
function removeWritingPopup() {
    const wiz_writing_popup = document.getElementById('wiz-writing-popup');
    if (wiz_writing_popup) {
        wiz_writing_popup.remove();
    }
}

// Function to handle text selection
function handleTextSelection() {
    removeWritingPopup();
    const selection = window.getSelection();
    if (selection.rangeCount > 0 && !selection.isCollapsed) {
        createWritingPopup(selection.getRangeAt(0));
    }
}

// Debounce function to limit the rate of function calls
function debounce(func, wait) {
    let timeout;
    return function executedFunction(...args) {
        const later = () => {
            clearTimeout(timeout);
            func(...args);
        };
        clearTimeout(timeout);
        timeout = setTimeout(later, wait);
    };
}

// Add event listeners
document.addEventListener('selectionchange', debounce(handleTextSelection, 400));

// Remove the wiz_writing_popup when clicking outside
document.addEventListener('mousedown', (e) => {
    const wiz_writing_popup = document.getElementById('wiz-writing-popup');
    if (wiz_writing_popup && !wiz_writing_popup.contains(e.target)) {
        removeWritingPopup();
    }
});

// Ensure the wiz_writing_popup is within viewport
function adjustWritingPopupPosition() {
    const wiz_writing_popup = document.getElementById('wiz-writing-popup');
    if (wiz_writing_popup) {
        const rect = wiz_writing_popup.getBoundingClientRect();
        if (rect.right > window.innerWidth) {
            wiz_writing_popup.style.left = `${window.innerWidth - rect.width}px`;
        }
        if (rect.bottom > window.innerHeight) {
            wiz_writing_popup.style.top = `${parseFloat(wiz_writing_popup.style.top) - rect.height - 10}px`;
        }
    }
}

// Handle window resize
window.addEventListener('resize', debounce(adjustWritingPopupPosition, 100));
